# Generated by roxygen2: do not edit by hand

S3method(contourview,"function")
S3method(contourview,Kriging)
S3method(contourview,WarpKriging)
S3method(contourview,character)
S3method(contourview,glm)
S3method(contourview,km)
S3method(contourview,list)
S3method(contourview,matrix)
S3method(filledcontourview,"function")
S3method(filledcontourview,Kriging)
S3method(filledcontourview,WarpKriging)
S3method(filledcontourview,glm)
S3method(filledcontourview,km)
S3method(filledcontourview,list)
S3method(sectionview,"function")
S3method(sectionview,Kriging)
S3method(sectionview,WarpKriging)
S3method(sectionview,character)
S3method(sectionview,glm)
S3method(sectionview,km)
S3method(sectionview,list)
S3method(sectionview,matrix)
S3method(sectionview3d,"function")
S3method(sectionview3d,Kriging)
S3method(sectionview3d,WarpKriging)
S3method(sectionview3d,character)
S3method(sectionview3d,glm)
S3method(sectionview3d,km)
S3method(sectionview3d,list)
S3method(sectionview3d,matrix)
export(Apply.function)
export(Memoize.function)
export(Vectorize.function)
export(are_in.mesh)
export(branin)
export(contourview)
export(expand.grids)
export(filledcontourview)
export(is.mesh)
export(is_in.mesh)
export(is_in.p)
export(mesh)
export(mesh_exsets)
export(mesh_level)
export(min_dist)
export(min_dist.mesh)
export(optim.stop)
export(optims)
export(plot2d_mesh)
export(plot3d_mesh)
export(plot_mesh)
export(points_in.mesh)
export(points_out.mesh)
export(root)
export(roots)
export(roots_mesh)
export(sectionview)
export(sectionview3d)
import(grDevices)
import(graphics)
import(methods)
import(stats)
importFrom(DiceDesign,lhsDesign)
importFrom(R.cache,loadCache)
importFrom(R.cache,saveCache)
importFrom(foreach,"%dopar%")
importFrom(foreach,foreach)
importFrom(geometry,convhulln)
importFrom(geometry,delaunayn)
importFrom(geometry,inhulln)
importFrom(parallel,mclapply)
importFrom(utils,combn)
importFrom(utils,installed.packages)
