[![R-CMD-check](https://github.com/IRSN/DiceView/actions/workflows/check-standard.yml/badge.svg)](https://github.com/IRSN/DiceView/actions/workflows/check-standard.yml)
[![submit-CRAN](https://github.com/IRSN/DiceView/actions/workflows/submit-on-cran.yml/badge.svg)](https://github.com/IRSN/DiceView/actions/workflows/submit-on-cran.yml)
[![CRAN-status](https://www.r-pkg.org/badges/version/DiceView)](https://cran.r-project.org/package=DiceView)
[![Downloads(monthly)](https://cranlogs.r-pkg.org/badges/DiceView)](https://cran.r-project.org/package=DiceView)

# DiceView: Methods for visualization of Computer Experiments Design and Surrogate

View 2D/3D sections, contour plots, mesh of excursion sets for computer experiments designs, surrogates or test functions.

Installation
------------

You can install the stabe version of the package using CRAN:

```
install.packages('DiceView')
```

Or install the latest version of the package using the `remotes` R package:

```
install.packages('remotes')
remotes::install_github("IRSN/DiceView")
```

![Analytics](https://ga-beacon.appspot.com/UA-109580-20/DiceView)
