% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mesh_set.R
\name{is.mesh}
\alias{is.mesh}
\title{Checks if a mesh is valid}
\usage{
is.mesh(x)
}
\arguments{
\item{x}{mesh to check}
}
\value{
TRUE if mesh is valid
}
\description{
Checks if a mesh is valid
}
